from distutils.core import setup, Extension
from distutils import sysconfig
from distutils.file_util import copy_file
import shutil, sys, os, os.path, glob
sys.path.append('./Lib')
import codegenerator

VERSION = "0.3"

if not hasattr(sys, 'version_info') or sys.version_info < (2,0,0,'alpha',0):
    raise SystemExit, "Python 2.0 or later required to build numarray."

EXTRA_LINK_ARGS = []
if sys.platform == "osf1V5":
    EXTRA_COMPILE_ARGS = ["-ieee"]
else:
    EXTRA_COMPILE_ARGS = []

print "Using EXTRA_COMPILE_ARGS =",EXTRA_COMPILE_ARGS

# sys.stderr = open("error.txt",'w')

#
# PseudoMake compares the modification date of Lib/codegenerator.py
# to the source code module implied by Name.  If the codegenerator is
# newer, the specified module is regenerated.
#
def PseudoMake(Name, Type=None):
    if Type is None:
        Source = "Src/_%smodule.c" % Name
    else:
        Source = "Src/_%smodule.c" % (Name+Type)
    timemodgen = os.path.getmtime("Lib/codegenerator.py")
    # If codegenerator module more recent, regenerate Source
    if os.path.exists(Source):
        timemod = os.path.getmtime(Source)
    else:
        timemod = -1
    if timemodgen > timemod:
        print "generating new version of", Source
        codegenerator.generate(Name, Type)

PseudoMake("conv")
PseudoMake("sort")
PseudoMake("bytes")
for T in "Bool Int8 UInt8 Int16 UInt16 Int32 Float32 Float64".split():
    PseudoMake("ufunc",T)

class OurExtension(Extension):
    """OurBaseExtension is an Extension with implicit include_dirs,
    extra_compile_args, and extra_link_args.  Used to construct our
    c-api shared library.
    """
    def __init__(self, module, Sources=None, lib_dirs=[], libs=[]):
        if Sources is None:
            Sources = [os.path.join("Src",module+"module.c")]
        Extension.__init__(self, module, Sources,
                           include_dirs=["Include/numarray"],
                           library_dirs=lib_dirs,
                           libraries=libs,
                           extra_compile_args=EXTRA_COMPILE_ARGS,
                           extra_link_args=EXTRA_LINK_ARGS)

def headers():
    return glob.glob(os.path.join("Include","numarray","*.h"))

#
# Run setup, but only through the stage specified as the parameter.
# Return the distribution object for access to the metadata.
#
def dosetup(how_far="run"):
    from distutils.core import _setup_stop_after
    _setup_stop_after = how_far
    return setup(name = "numarray",
                 version = VERSION,
                 description = \
                 """An array processing package modelled after Numeric with
                 improved type coercion rules and the ability to operate on
                 binary arrays in files.  Also supports viewing tables of
                 fixed length records as multiple arrays which run down the
                 table columns.  Tables can be processed "in-place" and hence
                 work efficiently with very large memory mapped files.
                 """,
                 url = "http://stsdas.stsci.edu/numarray/",
                 author = "Perry Greenfield",
                 author_email  = "perry@stsci.edu",
                 packages=[""],
                 package_dir={"":"Lib"},
                 extra_path="numarray",
                 headers = headers(),
                 ext_modules = map(OurExtension,
                                   ["_conv","_sort", "_bytes",
                                    "_ufuncBool",
                                    "_ufuncInt8",
                                    "_ufuncUInt8",
                                    "_ufuncInt16",
                                    "_ufuncUInt16",
                                    "_ufuncInt32",
                                    "_ufuncFloat32",
                                    "_ufuncFloat64",
                                    "_numarray", "_chararray",
                                    "memory", "libnumarray"])
                 )

#
# Add a command line parameter --local=<dir> which is an abbreviation for
# 'put all of numarray in <dir>/numarray'.
#
def dolocal():
    if "--help" in sys.argv:
        print >>sys.stderr
        print >>sys.stderr, "Numarray options:"
        print >>sys.stderr, "--local=<dir>   same as --install-lib=<dir> --install-headers=<dir>/numarray"
        print >>sys.stderr
    
    for a in sys.argv:
        if a.startswith("--local="):
            dir = a.split("=")[1]
            sys.argv.extend([
                "--install-lib="+dir,
                "--install-headers="+os.path.join(dir,"numarray")
                ])
            sys.argv.remove(a)            


dolocal()

#
#  Figure out where includes will go and create numinclude.py
#
if "install" in sys.argv and "--help" not in sys.argv:
    dist = dosetup("commandline")
    include_dir = dist.get_command_obj("install").install_headers
    include_dir = os.path.abspath(include_dir)
    f = open(os.path.join("Lib", "numinclude.py"), "w")
    f.write("# This file is generated by setup.py.  DO NOT EDIT.\n\n")
    f.write("include_dir = " + `include_dir` + "\n")
    f.write("version = " + `VERSION` + "\n")
    f.close()


#
# Do the installation.
#
dosetup()

